# Script for 3D cube with periodic boundary conditions 

from random import choice
from numarray import *
import os

#domain size: (horizNodes-1)x(vertNodes-1) elements
horizSize=1.
vertSize=1.
depthSize=1.
horizNodes=21
vertNodes=21
depthNodes=21
incX=1
incY=horizNodes
incZ=vertNodes*horizNodes
elementType='C3D8R'

#material properties
elasticity=100.0
poisson = 0.0
density=1.

nodeNumber=array(zeros((horizNodes,vertNodes,depthNodes),Int))
elemNumber=array(zeros((horizNodes-1,vertNodes-1,depthNodes-1),Int))
coordsX=array(zeros((horizNodes,vertNodes,depthNodes),Float))
coordsY=array(zeros((horizNodes,vertNodes,depthNodes),Float))
coordsZ=array(zeros((horizNodes,vertNodes,depthNodes),Float))
connectivity=array(zeros((horizNodes*vertNodes*depthNodes,8),Int))

#create the common geometry for the survay and initial files
def createGeometryFile(geometryFileName):
        g = open(geometryFileName, 'w')
	g.write('*NODE,NSET=NODES'+'\n')

	for iz in range(depthNodes):
		for iy in range(vertNodes):
			for ix in range(horizNodes):
				nodeNumber[ix,iy,iz]=1+(ix)*incX+(iy)*incY+(iz)*incZ
				coordsX[ix,iy,iz]=ix*horizSize/horizNodes
				coordsY[ix,iy,iz]=iy*vertSize/vertNodes
				coordsZ[ix,iy,iz]=iz*depthSize/depthNodes
				g.write(str(nodeNumber[ix,iy,iz])+',' +str(coordsX[ix,iy,iz])+','+str(coordsY[ix,iy,iz])+','+str(coordsZ[ix,iy,iz])+'\n')

        g.write('*ELEMENT,TYPE='+elementType+',ELSET=BOX'+'\n')
	for iz in range(depthNodes-1):
		for iy in range(vertNodes-1):
			for ix in range(horizNodes-1):
				elemNumber[ix,iy,iz]=1+(ix)*incX+(iy)*incY+(iz)*incZ
				node1=elemNumber[ix,iy,iz]+incZ
				node2=elemNumber[ix,iy,iz]+incX+incZ
				node3=elemNumber[ix,iy,iz]+incX
				node4=elemNumber[ix,iy,iz]
				node5=elemNumber[ix,iy,iz]+incZ+incY
				node6=elemNumber[ix,iy,iz]+incX+incZ+incY
				node7=elemNumber[ix,iy,iz]+incX+incY
				node8=elemNumber[ix,iy,iz]+incY
				connectivity[elemNumber[ix,iy,iz]]= [node1,node2,node3,node4,node5,node6,node7,node8]
				g.write(str(elemNumber[ix,iy,iz])+',' +str(node1)+','+str(node2)+','+str(node3)+','+str(node4)+','+str(node5)+','+str(node6)+','+str(node7)+','+str(node8)+'\n')

        g.write('*NSET,NSET=BOTTOM'+'\n')
	for ix in range(horizNodes):
		for iz in range(depthNodes):
			g.write(str(nodeNumber[ix,0,iz])+'\n')
	g.write('*NSET,NSET=TOP'+'\n')
	for ix in range(horizNodes):
		for iz in range(depthNodes):
			g.write(str(nodeNumber[ix,vertNodes-1,iz])+'\n')
        g.write('*NSET,NSET=LEFT'+'\n')
	for iy in range(vertNodes):
        	for iz in range(depthNodes):
			g.write(str(nodeNumber[0,iy,iz])+'\n')
        g.write('*NSET,NSET=RIGHT'+'\n')
	for iy in range(vertNodes):
		for iz in range(depthNodes):
			g.write(str(nodeNumber[horizNodes-1,iy,iz])+'\n')
	g.write('*NSET,NSET=FRONT'+'\n')
	for ix in range(horizNodes):
		for iy in range(vertNodes):
			g.write(str(nodeNumber[ix,iy,0])+'\n')
	g.write('*NSET,NSET=BEHIND'+'\n')
	for ix in range(horizNodes):
		for iy in range(vertNodes):
			g.write(str(nodeNumber[ix,iy,depthNodes-1])+'\n')
			
def createRheologyFile(rheologyFileName):
        r = open(rheologyFileName, 'w')
	r.write('*MATERIAL,NAME=HyperElastic'+'\n')
        r.write('*ELASTIC, TYPE=ISO \n')
	r.write(str(elasticity)+','+ str(poisson)+',,\n')
	r.write('*DENSITY\n')
	r.write(str(density)+'\n')
        r.close()

def createInitialConditions(initialFileName):
	f = open(initialFileName, 'w')
	f.write('*INITIAL CONDITIONS, TYPE=VELOCITY \n')
        for iy in range(vertNodes):
        	for ix in range(horizNodes):
			for iz in range(depthNodes):
				initialVelocity=sin(2.*pi*coordsX[ix,iy,iz])
				f.write(str(nodeNumber[ix,iy,iz])+',1,'+ str(initialVelocity)+'\n')
				f.write(str(nodeNumber[ix,iy,iz])+',2,0\n')
				f.write(str(nodeNumber[ix,iy,iz])+',3,0\n')
	f.close()
	
def createBoundaryConditions(boundaryFileName):
	f = open(boundaryFileName, 'w')
	f.write('*EQUATION \n')

        for iy in range(vertNodes):
        	for ix in range(horizNodes):
				f.write(str(nodeNumber[ix,iy,0])+',1,-1,'+ str(nodeNumber[ix,iy,depthNodes-1])+',1,1\n')
				f.write(str(nodeNumber[ix,iy,0])+',2,-1,'+ str(nodeNumber[ix,iy,depthNodes-1])+',2,1\n')

        for iz in range(depthNodes):
        	for ix in range(horizNodes):
				f.write(str(nodeNumber[ix,0,iz])+',1,-1,'+ str(nodeNumber[ix,vertNodes-1,iz])+',1,1\n')
				f.write(str(nodeNumber[ix,0,iz])+',3,-1,'+ str(nodeNumber[ix,vertNodes-1,iz])+',3,1\n')

	for iz in range(depthNodes):
        	for iy in range(vertNodes):
				f.write(str(nodeNumber[0,iy,iz])+',2,-1,'+ str(nodeNumber[horizNodes-1,iy,iz])+',2,1\n')
				f.write(str(nodeNumber[0,iy,iz])+',3,-1,'+ str(nodeNumber[horizNodes-1,iy,iz])+',3,1\n')	
	f.close()

def createInputFile(inputFileName,geometryFileName,initialFileName):
	i = open(inputFileName, 'w')
	i.write('*HEADING'+'\n')
        i.write('*INCLUDE,INPUT='+geometryFileName+'\n')
        i.write('*SOLID SECTION, ELSET=BOX, MATERIAL=HyperElastic'+'\n')
        i.write('*INCLUDE,INPUT='+rheologyFileName+'\n')
	i.write('*INCLUDE,INPUT='+initialFileName+'\n')
        i.write('*BOUNDARY, OP=NEW, TYPE=DISPLACEMENT'+'\n')
	i.write('LEFT, 1,1,0.\n')
	i.write('RIGHT, 1,1,0.\n')
	i.write('BOTTOM, 2,2,0.\n')
	i.write('TOP, 2,2,0.\n')
	i.write('FRONT,3,3,0.\n')
	i.write('BEHIND,3,3,0.\n')
	i.write('*STEP, NLGEOM=YES'+'\n')
	i.write('*DYNAMIC,EXPLICIT,')
	#i.write(','+str(totalTime)+'\n')
	i.write('DIRECT USER CONTROL'+'\n')
	i.write('0.001 ,'+str(totalTime)+'\n')
        i.write('*Output, field, op=NEW, NUMBER INTERVAL=200'+'\n')
	i.write('*Node Output,NSET=NODES'+'\n')
	i.write('U, V'+'\n')
	i.write('*Element Output,ELSET=BOX'+'\n')
	i.write('LE, ER, S'+'\n')
	i.write('*OUTPUT, HISTORY, OP=NEW'+'\n')
	i.write('*ENERGY OUTPUT'+'\n')
	i.write('ALLAE, ALLCD, ALLFD, ALLIE, ALLKE, ALLPD, ALLSE, ALLVD, ALLWK, ETOTAL'+'\n')
	i.write('*END STEP'+'\n')


# HERE THE PROGRAM BEGINS
#create a separate file with the geometry
totalTime=.2 # 3Myrs
geometryFileName='geometry.inp'
createGeometryFile(geometryFileName)
initialFileName='initialConditions.inp'
createInitialConditions(initialFileName)
rheologyFileName='rheology.inp'
createRheologyFile(rheologyFileName)
refName='3dCube'
inputFileName=refName+'.inp'
createInputFile(inputFileName,geometryFileName,initialFileName)
os.system('abaqus job='+refName+' double int \n')
print refName + ' done'



